/** 
	A "hello" program in Qt
	that use OpenGL drawing primitives
	File: quad.cpp

	Fraga 15/09/2016
**/
#include <qpainter.h>
#include <ctype.h>
#include "aspas.h"

Aspas::Aspas( QWidget *parent )
		: QGLWidget( parent )
//  : QGLWidget( QGLFormat( QGL::SingleBuffer|QGL::NoDepthBuffer ), parent )
{
	signo = 1;	

	// setMouseTracking( TRUE );

	// setFocusPolicy( QWidget::StrongFocus);
	setFocusPolicy( Qt::StrongFocus);

	timer = new QTimer( this );
	connect ( timer, SIGNAL(timeout()), this, SLOT(timerDone()) );

	xrot = yrot = 0;
	angle = 0;

	setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );

	crea_Aspa( a1, 2, 0.6, 0.05 );
}


void Aspas::timerDone( )
{
	angle += 10.0*signo;

	if ( angle >= 360.0 ) angle = 0.0;
	if ( angle <= -360.0 ) angle = 0.0;
	
	updateGL( );
}

void Aspas::mouseMoveEvent( QMouseEvent * e )
{
	QPoint c = e->pos( );

	xmouse = c.x( );
	ymouse = c.y( );
	if ( xmouse>=0 && ymouse>=0 )  emit changePos ( );

	yrot = (float)xmouse*360.f/(float)windowWidth - 180.f;
	xrot = (float)ymouse*360.f/(float)windowHeight - 180.f;

	updateGL( );
}

void Aspas::keyPressEvent ( QKeyEvent * e )
{
	switch ( e->key() ) {
		case '+':   
			signo = 1;
			break;
		case '-':  
			signo = -1;
			break;
	}
}

//
// This method puts the initial coordinates each time window is resized
//
void Aspas::resizeGL( int l, int a )
{
	float aspecto=(float)l/(float)a;

	windowWidth = l;
	windowHeight = a;

	glViewport (0, 0, (GLsizei) l, (GLsizei) a);

	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();

	if( l >= a )
		glOrtho(-4.0*aspecto, 4.0*aspecto, -4.0, 4.0, -4.0, 4.0 );
	else
		glOrtho( -4.0, 4.0, -4.0/aspecto, 4.0/aspecto, -4.0, 4.0 );
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();			 /* clear the matrix */

	emit changeSize ( );
}

QSizePolicy Aspas::sizePolicy() const
{
	return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
}

void Aspas::paintGL(void)
{
	/** Clear all pixels **/
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// Se obtiene una copia de la matrix identidad I
	glPushMatrix( );

	glRotatef ( xrot , 1.0, 0.0, 0.0);

	glRotatef ( angle , 0.0, 1.0, 0.0);

	dibuja_Aspa( a1 );

		glPushMatrix( );
		glRotatef( 120.0, 0.0, 1.0, 0.0 );
		dibuja_Aspa( a1 );
		glPopMatrix( );

		glPushMatrix( );
		glRotatef( -120.0, 0.0, 1.0, 0.0 );
		dibuja_Aspa( a1 );
		glPopMatrix( );

	glPopMatrix( );

	// swapBuffers( );
}

void Aspas::initializeGL( ) 
{
	glClearColor ( 0.8, 0.8, 0.8, 0.0);	// Background to a grey tone

	/* initialize viewing values  */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0 );

	glEnable (GL_DEPTH_TEST);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();			 /* clear the matrix */
}

/**
  a = largo del aspa
  b = mitad del ancho del aspa
  c = mitad del grueso del aspa
**/
void Aspas::crea_Aspa( float v[][3], float a, float b, float c )
{
	v[0][0] = 0.0; v[0][1] = 0.0; v[0][2] = 0.0;

	v[1][0] = a;   v[1][1] = c;   v[1][2] = -b;
	v[2][0] = a;   v[2][1] = c;   v[2][2] = b;

	v[3][0] = a;   v[3][1] = -c;  v[3][2] = b;
	v[4][0] = a;   v[4][1] = -c;  v[4][2] = -b;
}

void Aspas::dibuja_Aspa( float v[][3] )
{
	int i;
	static int c[6][3] = { {0,1,2}, {0,3,4}, {3,0,2},
                         {0,4,1}, {1,4,2}, {4,3,2} };

	static float color[6][3] = 
       { { 1.0, 1.0, 1.0},
         { 1.0, 1.0, 1.0},
         { 0.7, 0.7, 0.7},
         { 0.7, 0.7, 0.7},
         { 0.0, 0.0, 0.0},
         { 0.0, 0.0, 0.0} };

	glBegin( GL_TRIANGLES );

	for( i=0; i<6; i++ ) {
		glColor3fv( &color[i][0] );
		glVertex3fv( &v[ c[i][0] ][0] );
		glVertex3fv( &v[ c[i][1] ][0] );
		glVertex3fv( &v[ c[i][2] ][0] );
	}

	glEnd( );
}
